<?php
/**
 * Watson Base Class.
 *
 * $Horde: watson/lib/Skeleton.php,v 1.17 2007/02/21 10:25:28 jan Exp $
 *
 * Copyright 2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Jan Zagar <jan.zagar@siol.net>
 * @package Watson
 */
class Watson {

   /**
    * Constructs driver for selected channel.
    *
    * @param string $channel    Channel name.
    * @param string $date       Date.
    */
    function factory($channel, $date, $driver = null)
    {
        if ($driver === null) {
            $driver = Watson::getDriver($channel);
            if (empty($driver)) {
                return false;
            }
        }
        $driver = basename($driver);

        $class = 'Watson_' . $driver;
        if (!class_exists($class)) {
            include dirname(__FILE__) . '/Driver/' . $driver . '.php';
        }
        if (class_exists($class)) {
            return new $class($channel, $date);
        } else {
            return false;
        }
    }

    /**
     * Inserts schedule into database or reads from database.
     */
    function process()
    {
        $storage = Watson_Driver::factory();
        $storage->retrieve($this->channel, $this->date);

        if (!empty($storage->_schedule) && is_array($storage->_schedule)) {
            return $storage->_schedule;
        } else {
            $result = $this->parse();
            if ($result instanceof PEAR_Error) {
                return $result;
            } else {
                $storage->insert($this->channel, $this->date, $result);
            }
            return $result;
        }
    }

   /**
    * Generates source page url from config/sources.php.
    *
    * @return string    Url.
    */
    function parameters()
    {
        require WATSON_BASE . '/config/sources.php';
        $domain = $chan[$this->channel]['domain'];
        $file = $chan[$this->channel]['file'];
        $parameters = '?';

        if (isset($chan[$this->channel]['parameters'])) {
            foreach ($chan[$this->channel]['parameters'] as $key => $value) {
                $parameters .= $key . '=' . $value . '&';
            }
        }

        if (isset($chan[$this->channel]['DATE'])) {
            $key = key($chan[$this->channel]['DATE']);
            $format = $chan[$this->channel]['DATE'][$key];
            $parameters .= $key . '=' . $this->getDate($this->date, $format);
        }

        return 'http://' . $domain . '/' . $file . $parameters;
    }

    /**
    * Converts date into choosen date format, or into timestamp.
    *
    * @param string $date    Date.
    * @param string $format  Date format.
    */
    function getDate($date = '', $format = 'Y-m-d')
    {
        if (empty($date)) {
            $date = date('Y-m-d');
        }

        if ($format == 'timestamp') {
            return strtotime($date);
        }

        list($year, $month, $day) = explode('-', $date);
        return date($format, mktime(0, 0, 0, $month, $day, $year));
    }

   /**
    * Returns driver for selected channel.
    *
    * @param string $channel    Channel.
    *
    * @return string            Returns driver.
    */
    function getDriver($channel)
    {
        require WATSON_BASE . '/config/sources.php';
        if (empty($chan[$channel]['driver'])) {
            return null;
        }

        return $chan[$channel]['driver'];
    }

   /**
    * Loads xml source page
    *
    * @return dom document object
    */
    function xmlSource()
    {
        global $chan;

        $this->cache_expires = $chan[$this->channel]['cache_expires'];
        $doc = $GLOBALS['cache']->get($this->cache, $this->cache_expires);
        $doc = false;

        if ($doc === false) {

            $doc = new DOMDocument;

            if (ini_get('allow_url_fopen')) {

                $load = @$doc->load($this->parameters());

            } else {

                $tmp_name = Horde::getTempFile();
                $ch = curl_init($this->parameters());
                if ($fp = fopen($tmp_name, 'w')) {

                    curl_setopt($ch, CURLOPT_FILE, $fp);
                    curl_setopt($ch, CURLOPT_HEADER, 0);
                    curl_exec($ch);
                    curl_close($ch);
                    fclose($fp);

                    $content = file_get_contents($tmp_name);
                    $load = @$doc->loadXml($content);
                }
            }
        }

        if ($load) {
            return $doc;
        } else {
            return PEAR::raiseError(sprintf(_("Cannot read input url %s."), $this->parameters()));
        }
    }

    /**
    * Loads html source page
    *
    * @return dom document object
    */
    function htmlSource()
    {
        global $chan;

        $this->cache_expires = $chan[$this->channel]['cache_expires'];
        $doc = $GLOBALS['cache']->get($this->cache, $this->cache_expires);
        $doc = false;

        if ($doc === false) {

            $doc = new DOMDocument;

            if (ini_get('allow_url_fopen')) {

                $load = @$doc->loadHTMLFile($this->parameters());

            } else {

                $tmp_name = Horde::getTempFile();
                $ch = curl_init($this->parameters());
                if ($fp = fopen($tmp_name, 'w')) {

                    curl_setopt($ch, CURLOPT_FILE, $fp);
                    curl_setopt($ch, CURLOPT_HEADER, 0);
                    curl_exec($ch);
                    curl_close($ch);
                    fclose($fp);

                    $content = file_get_contents($tmp_name);
                    $load = @$doc->loadHTML($content);
                }
            }
        }

        if ($load) {
            return $doc;
        } else {
            return PEAR::raiseError(sprintf(_("Cannot read input url %s."), $this->parameters()));
        }
    }

    /**
    * Parse xml or html from source page.
    */
    function parse()
    {
        return PEAR::raiseError(_("Timeline for this channel/day does not exist."));
    }

    /**
    * Displays the schedule
    * @param array $result    Schedule in array, example:
    *                         $result[time] = title; 
    */
    function display($result)
    {
        if (isset($this->cache) && isset($this->cache_expires)) {
            $print = $GLOBALS['cache']->get($this->cache, $this->cache_expires);
        } else {
            $print = false;
        }

        if ($result instanceof PEAR_Error) {
            return $result->getMessage();
        }

        if ($print === false) {
            $print = '<div id="watson"><table>' . "\n";

            foreach ($result as $time => $title) {
            $time = htmlspecialchars($time);
            $title = htmlspecialchars($title);
            if ($time != 'channel' && $time != 'date' && $time != 'description' && $time != 'forecast') {

                $print .= '<tr><td class="time">' . $time . '</td><td class="title">' . $title;

                if (isset($result['description'][$time])) {
                    $print .= '<div class="description">' . _("Description") . ': ' . $result['opis'][$ura] . '</div>';
                }

                if (isset($result['forecast'][$time])) {
                    $print .= '<div class="forecast">' . _("Forecast") . ': ' . $result['forecast'][$time] . '</div></a>';
                }

                $print .= '</td></tr>' . "\n";
            }
                }
            $print .= '</table></div>' . "\n";
            $GLOBALS['cache']->set($this->cache, $print);
        }

        return $print;
    }

    /**
    * Prepare chanel selection array
    */
    function getChannelSelection()
    {
        require WATSON_BASE . '/config/sources.php';

        $values = array();
        foreach ($chan as $channel_name => $value) {
            $values[$channel_name] = $channel_name;
        }
        natcasesort($values);

        return $values;
    }

    /**
     * Builds Watson's list of menu items.
     */
    function getMenu($returnType = 'object')
    {
        require_once 'Horde/Menu.php';

        $dir = $GLOBALS['registry']->getImageDir('horde');

        $menu = new Menu(HORDE_MENU_MASK_ALL);
        $menu->add(Horde::applicationUrl('fetch.php'), _("Date"), 'calendar.png', $dir);
        $menu->add(Horde::applicationUrl('content.php'), _("Today shows"), 'layout.png', $dir);
        $menu->add(Horde::applicationUrl('now.php'), _("Today timetable"), 'show_panel.png', $dir);
        $menu->add(Horde::applicationUrl('current.php'), _("Current schedule"), 'info.png', $dir);

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }

    }
}